import { bindActionCreators } from 'redux';
import withFit from './fit-element';
import { isFunction } from './util';
// We declare the exported typings separately for now (above).
export default function connect(mapStateToProps, mapDispatchToProps) {
    return (b) => {
        const base = isTemplateFunction(b) ? withFit(b)(HTMLElement) : b;
        return class extends base {
            constructor(...args) {
                super(...args);
                this._propsEnqueued = false;
                this._preparedMapStateToProps = isFactory(mapStateToProps)
                    ? mapStateToProps()
                    : mapStateToProps;
            }
            get ownProps() {
                return this._ownProps;
            }
            set ownProps(props) {
                this._ownProps = props;
                this._computeProps();
            }
            connectedCallback() {
                super.connectedCallback();
                const store = this.getStore();
                this._preparedDispatch = isFunction(mapDispatchToProps)
                    ? mapDispatchToProps
                    : bindActionCreators(mapDispatchToProps, store.dispatch);
                this._unsubscribe = store.subscribe(() => this._computeProps());
                this._computeProps();
            }
            disconnectedCallback() {
                super.disconnectedCallback();
                this._unsubscribe();
                this._store = undefined;
            }
            getStore() {
                if (this._store) {
                    return this._store;
                }
                if (isFunction(super.getStore)) {
                    return super.getStore();
                }
                let node = this;
                while (node = node.parentNode || node.host) {
                    if (isFunction(node.getStore)) {
                        this._store = node.getStore();
                        return this._store;
                    }
                }
                throw new Error("💪-html: Missing redux store.\nSeems like you're using fit-html without a redux store. Please use a provider component to provide one to the element tree.");
            }
            _computeProps() {
                if (this._propsEnqueued) {
                    return;
                }
                this._propsEnqueued = true;
                Promise.resolve().then(() => {
                    this._propsEnqueued = false;
                    const store = this.getStore();
                    const props = this._preparedMapStateToProps(store.getState(), this.ownProps);
                    const dispatch = isFunction(this._preparedDispatch)
                        ? this._preparedDispatch(store.dispatch, this.ownProps)
                        : this._preparedDispatch;
                    this.renderProps = Object.assign({}, props, dispatch);
                });
            }
        };
    };
}
/* tslint:enable */
function isFactory(fn) {
    return fn.length === 0;
}
function isTemplateFunction(base) {
    return !(base.prototype instanceof HTMLElement);
}
//# sourceMappingURL=connect.js.map